#version 120

#define RESOLUTION vec2(1920, 1080)


uniform sampler2D       Texture;
uniform sampler2D       Scene;
uniform sampler2D       Dirt;
uniform vec4            Param1; // (SunPosX, SunPosY, 1.0/fbo.width, 0.5/FlareTexture.width)
uniform vec4            Param2; // (Radius, Stride, Bright, Scale)
uniform float			screenRatio; // GLFrame.getHeight() / GLFrame.getWidth()

uniform sampler2D sam;

const float exposure = 0.0;
const float decay = 0.5;
const float density = 3.0;
const float weight = 0.25;
const float weight2 = 0.125;
const float weight3 = 0.1333;
const float weight4 = 0.0733;


const float Sexposure = 0.67;
const float Sdecay = 0.996;
const float Sdensity = 0.8;
const float Sweight = 0.02;
/*
float Density = 3;
float Exposure = -.1;
float Weight = .25;
float Weight2 = .125;
float Decay = .5;
*/

float snoise(vec3 uv, float res)
{
	const vec3 s = vec3(1e0, 1e2, 1e4);
	
	uv *= res;
	
	vec3 uv0 = floor(mod(uv, res))*s;
	vec3 uv1 = floor(mod(uv+vec3(1.), res))*s;
	
	vec3 f = fract(uv); f = f*f*(3.0-2.0*f);
	
	vec4 v = vec4(uv0.x+uv0.y+uv0.z, uv1.x+uv0.y+uv0.z,
		      	  uv0.x+uv1.y+uv0.z, uv1.x+uv1.y+uv0.z);
	
	vec4 r = fract(sin(v*1e-3)*1e5);
	float r0 = mix(mix(r.x, r.y, f.x), mix(r.z, r.w, f.x), f.y);
	
	r = fract(sin((v + uv1.z - uv0.z)*1e-3)*1e5);
	float r1 = mix(mix(r.x, r.y, f.x), mix(r.z, r.w, f.x), f.y);
	
	return mix(r0, r1, f.z)*2.-1.;
}

//uniform vec2 lightPositionOnScreen;
uniform sampler2D firstPass;
const int NUM_SAMPLES = 30 ;
	
	vec4 DoLenseFlare(vec2 ScreenLightPosition,vec2 texCoord,bool fwd, float weight, float xTC, float dist)
{


    // Calculate vector from pixel to light source in screen space.
    vec2 deltaTexCoord = (texCoord - ScreenLightPosition.xy);// Divide by number of samples and scale by control factor.
    
     
    deltaTexCoord *= (dist) /  density;
    
    //deltaTexCoord *= screenRatio;
 
 		deltaTexCoord.x /= screenRatio;
 
    // Store initial sample.
    vec3 color = vec3(0.0);
 
    // Set up illumination decay factor.
    float illuminationDecay = 1.0f;
 
    for (int i = 0; i < 3 ; i++)
    {
    
    	vec3  core = texture2D(firstPass, ScreenLightPosition.xy  - vec2(float(i)/3.0,float(i)/3.0) * Param2.w).rgb;
	            
        float bright = dot(core, vec3(0.333));
    
	        // Step sample location along ray.
	        if(fwd)
	            texCoord -= deltaTexCoord;
	        else
	            texCoord += deltaTexCoord;
	 
		 	if(texCoord.y > 1.0 || texCoord.y < 0.0 || texCoord.x > 1.0 || texCoord.x < 0.0){
		 		continue;
		 	}
		 	
	 		vec2 flareTC = texCoord;
	 		//flareTC.x /= screenRatio;
	 		flareTC.x *= 0.25;
	 		flareTC.x += xTC;
	 		
	 		
	 		
	        // Retrieve sample at new location.
	        vec4 sample = texture2D(Texture, flareTC);
	 
	        // Apply sample attenuation scale/decay factors.
	        sample *= illuminationDecay * weight;
	        
	 
	        // Accumulate combined color.
	        color += sample.xyz * bright ;
	 
	        // Update exponential decay factor.
	        illuminationDecay *= decay;
    }
 
    return vec4(color,1);
}
	
	
	
	vec4 addGodRay()
	{
	    vec2 flareTC = (gl_TexCoord[0].st - vec2(0.5));
	    
	    
	   // vec2 flareTC = (gl_TexCoord[0].st - vec2(0.5)) * Param1.z;
	    
	    
	    vec3 conv = vec3(0.0);
	
	     for (float x=-Param2.x; x<=Param2.x; x+=Param2.y)
	     {
	         for (float y=-Param2.x; y<=Param2.x; y+=Param2.y)
	         {
	            vec3  core = texture2D(firstPass, Param1.xy  - vec2(x,y) * Param2.w).rgb;
	            
	            float bright = dot(core, vec3(0.333));
	            if (bright > 0.99){
		            vec3 flare = texture2D(Texture,  flareTC + vec2(x,y) * Param1.w).rgb;
		            conv += flare*core;
	            }
	        }
	    }
		vec4 frag;
		frag.rbg = conv * Param2.z;
		frag.a = 0.0;
	   

    		//frag.rgb += texture2D(Scene, ((gl_TexCoord[0].xy ) * Param2.w * 800.0) -Param1.xy).rgb;
    		
    	return frag;
	}
	
	void main() {
	
	
		vec2 deltaTextCoord = vec2( gl_TexCoord[0].st - Param1.xy );
		vec2 textCoo = gl_TexCoord[0].st;
		
		deltaTextCoord *= 1.0 / float(NUM_SAMPLES) * Sdensity;
		
		float illuminationDecay = 1.0;
		
		vec4 fragCol = vec4(0.0,0.0,0.0,0.0);
		for(int i=0; i < NUM_SAMPLES ; i++) {
		
			textCoo -= deltaTextCoord;
			vec4 sample = texture2D(firstPass, textCoo );
			
			sample *= illuminationDecay * Sweight;
			fragCol += sample;
			illuminationDecay *= Sdecay;
			
		}
		
		
		vec2 coord = gl_TexCoord[0].st;//0.5 - (gl_TexCoord[0].st - Param1.xy)  * 0.5;
		
		;//texture2D(firstPass, textCoo );
		
		vec4 lf = DoLenseFlare(	Param1.xy,	coord,		true, 	weight, 	0.0,  3.0);
		lf += DoLenseFlare(		Param1.xy,	coord,		false, 	weight2, 	0.75, 4.0);
		lf += DoLenseFlare(		Param1.xy,	coord,		true, 	weight3, 	0.50, 11.0);
		lf += DoLenseFlare(		Param1.xy,	coord,		false, 	weight4, 	0.25, 12.0);
		
		vec4 col = ( lf * exposure) * 5.0;
		
		
		vec4 dirt = col*texture2D(Dirt, -gl_TexCoord[0].st).rrrr*2.0;
		
		fragCol += col;
		
		float brightness	= 0.25 * 0.25 + 0.1;
		float radius		= 1.4 + brightness * 0.2;
		float invRadius 	= 1.0/radius;
		
		vec3 orange			= vec3( 0.8, 0.65, 0.3 );
		vec3 orangeRed		= vec3( 0.8, 0.35, 0.1 );
		float time		=  0.1;
		float aspect	= screenRatio;
		vec2 uv			= gl_FragCoord.xy / vec2(RESOLUTION.x, RESOLUTION.y);
		vec2 p 			= -0.5 + uv;
		p.x *= aspect;
		//p *= 0.3;
		
		float angle		= atan( p.x, p.y )/6.2832;
		float dist		= length(p);

		vec2 sp = -1.0 + 2.0 * uv;
		sp.x *= aspect;
		sp *= ( 2.0 - brightness );
		float r = dot(sp,sp);
		float f = (1.0-sqrt(abs(1.0-r)))/(r) + brightness * 0.5;
		
		float starGlow	= min( max( 1.0 - dist * ( 1.0 - brightness ), 0.0 ), 1.0 );
		vec3 starCol	= vec3( f * ( 0.75 + brightness * 0.3 ) * orange ) * orange + starGlow * orangeRed;
		
		vec4 starCol4 = vec4(starCol + vec3(0.2, 0., 0.), 1.0);
		
		gl_FragColor = (fragCol*(starCol4));//+dirt; 
		//gl_FragColor *= exposure;
	}
